@@ -60,7 +60,7 @@ class NodeManagementSession(ApplicationSession):
         extra = {
             # forward the client pubkey: this allows us to omit authid as
             # the router can identify us with the pubkey already
-            'pubkey': self.config.extra['node_key'].public_key(),
+            'pubkey': self.config.extra['node'].secmod[1].public_key(binary=False),
 
             # not yet implemented. a public key the router should provide
             # a trustchain for it's public key. the trustroot can eg be
@@ -97,11 +97,14 @@ class NodeManagementSession(ApplicationSession):
             # is fine - the router is authentic wrt our trustroot.
 
             # sign the challenge with our private key.
-            signed_challenge = self.config.extra['node_key'].sign_challenge(self, challenge)
+            channel_id_type = self.config.extra.get('channel_binding', None)
+            channel_id = self.transport.transport_details.channel_id.get(channel_id_type, None)
+            signed_challenge = self.config.extra['node'].secmod[1].sign_challenge(challenge,
+                                                                                  channel_id=channel_id,
+                                                                                  channel_id_type=channel_id_type)
 
             # send back the signed challenge for verification
             return signed_challenge
-
         else:
             raise Exception(
                 'internal error: we asked to authenticate using wamp-cryptosign, but now received a challenge for {}'.
@@ -210,7 +213,6 @@ class NodeManagementBridgeSession(ApplicationSession):
         self._manager = manager
         self._management_realm = management_realm
         self._node_id = node_id
-        self._node_key = self.config.extra['node_key']
         self._controller_config = self.config.extra['controller_config']
 
         fabric = self._controller_config.get('fabric', {})
@@ -229,11 +231,10 @@ class NodeManagementBridgeSession(ApplicationSession):
         reactor.callLater(self._heartbeat_startup_delay, self._start_cfc_heartbeat)
 
         self.log.info(
-            '{klass}.attach_manager: manager attached as node "{node_id}" on management realm "{management_realm}") with public key "{public_key}"',
+            '{klass}.attach_manager: manager attached as node "{node_id}" on management realm "{management_realm}")',
             klass=self.__class__.__name__,
             node_id=self._node_id,
-            management_realm=self._management_realm,
-            public_key=self._node_key.public_key())
+            management_realm=self._management_realm)
         self.log.info('Controller configuration: {controller_config}', controller_config=self._controller_config)
 
     @inlineCallbacks
@@ -309,14 +310,12 @@ class NodeManagementBridgeSession(ApplicationSession):
                     del status[k]
 
         if self._manager and self._manager.is_attached():
-            node_pubkey = str(self._node_key.public_key())
 
             # get basic status
             status = yield self.call('crossbar.get_status')
             obj = {
                 'timestamp': self._heartbeat_time_ns,
                 'period': self._heartbeat_heartbeat_period,
-                'pubkey': node_pubkey,
                 'mrealm_id': self._management_realm,
                 'seq': self._heartbeat,
                 'workers': status.get('workers_by_type', {}),
@@ -394,7 +393,6 @@ class NodeManagementBridgeSession(ApplicationSession):
                         _drop_attr(worker_status)
                         worker_status['timestamp'] = self._heartbeat_time_ns
                         worker_status['period'] = self._heartbeat_heartbeat_period
-                        worker_status['pubkey'] = node_pubkey
                         worker_status['mrealm_id'] = self._management_realm
                         worker_status['seq'] = self._heartbeat
                         worker_status['type'] = worker_status['process']['type']
