@@ -7,7 +7,7 @@
 
 import uuid
 from pprint import pformat
-from typing import Optional, List, Dict
+from typing import Optional, List, Dict, Tuple
 
 import numpy as np
 
@@ -16,13 +16,14 @@ from autobahn.wamp.exception import ApplicationError
 from autobahn.wamp.types import CallDetails, PublishOptions, RegisterOptions
 
 from crossbar.common import checkconfig
-from crossbar.webservice import archive
+from crossbar.webservice import archive, wap
 from crossbar._util import hl, hlid, hltype, hlval, get_free_tcp_port
 from cfxdb.mrealm import WebCluster, WebClusterNodeMembership, WebService
 from cfxdb.mrealm import cluster
 from cfxdb.mrealm.application_realm import ApplicationRealmStatus
 
 import txaio
+
 txaio.use_twisted()
 from txaio import time_ns, sleep, make_logger  # noqa
 from twisted.internet.defer import inlineCallbacks
@@ -50,21 +51,39 @@ class WebClusterMonitor(object):
 
     @property
     def is_started(self):
+        """
+
+        :return:
+        """
         return self._loop is not None and self._loop.running
 
     def start(self):
+        """
+
+        :return:
+        """
         assert self._loop is None
 
-        self._loop = LoopingCall(self.check_and_apply)
+        self._loop = LoopingCall(self._check_and_apply)
         self._loop.start(self._interval)
 
     def stop(self):
+        """
+
+        :return:
+        """
         assert self._loop is not None
+
         self._loop.stop()
         self._loop = None
         self._check_and_apply_in_progress = False
 
-    def get_cluster_workers(self, filter_online=False):
+    def get_cluster_workers(self, filter_online: bool = False) -> List[Tuple[str, str]]:
+        """
+
+        :param filter_online:
+        :return:
+        """
         res = []
         for node_oid, worker_id in self._workers:
             if not filter_online or self._workers[(node_oid, worker_id)]['status'] == 'started':
@@ -72,21 +91,19 @@ class WebClusterMonitor(object):
         return res
 
     @inlineCallbacks
-    def check_and_apply(self):
-        """
-
-        :return:
-        """
+    def _check_and_apply(self):
         if self._check_and_apply_in_progress:
+            # we prohibit running the iteration multiple times concurrently. this might
+            # happen when the iteration takes longer than the interval the monitor is set to
             self.log.info('{func} {action} for webcluster {webcluster} skipped! check & apply already in progress.',
                           action=hl('check & apply run skipped', color='red', bold=True),
-                          func=hltype(self.check_and_apply),
+                          func=hltype(self._check_and_apply),
                           webcluster=hlid(self._webcluster_oid))
             return
         else:
             self.log.info('{func} {action} for webcluster {webcluster} ..',
                           action=hl('check & apply run started', color='green', bold=True),
-                          func=hltype(self.check_and_apply),
+                          func=hltype(self._check_and_apply),
                           webcluster=hlid(self._webcluster_oid))
             self._check_and_apply_in_progress = True
 
@@ -111,7 +128,7 @@ class WebClusterMonitor(object):
 
                 if membership.standby:
                     self.log.debug('{func} Web cluster node {node_oid} is configured for standby',
-                                   func=hltype(self.check_and_apply),
+                                   func=hltype(self._check_and_apply),
                                    node_oid=hlid(node_oid))
                     continue
 
@@ -120,7 +137,7 @@ class WebClusterMonitor(object):
 
                 if node and node.status == 'online':
                     self.log.debug('{func} Ok, web cluster node {node_oid} is running!',
-                                   func=hltype(self.check_and_apply),
+                                   func=hltype(self._check_and_apply),
                                    node_oid=hlid(node_oid))
 
                     # we expect "parallel" workers to run on this node ..
@@ -131,7 +148,7 @@ class WebClusterMonitor(object):
 
                         self.log.debug(
                             '{func} Performing checks for configured proxy worker {worker_index}/{parallel} [{worker_id}] ..',
-                            func=hltype(self.check_and_apply),
+                            func=hltype(self._check_and_apply),
                             worker_index=hlid(worker_index + 1),
                             worker_id=hlid(worker_id),
                             parallel=hlid(membership.parallel))
@@ -152,7 +169,7 @@ class WebClusterMonitor(object):
                         else:
                             self.log.debug(
                                 '{func} Ok, web cluster worker {worker_id} already running on node {node_oid}!',
-                                func=hltype(self.check_and_apply),
+                                func=hltype(self._check_and_apply),
                                 node_oid=hlid(node_oid),
                                 worker_id=hlid(worker_id))
 
@@ -168,7 +185,7 @@ class WebClusterMonitor(object):
                                     'crossbarfabriccenter.remote.node.get_worker', node_oid, worker_id)
                                 self.log.info(
                                     '{func} Web cluster worker {worker_id} started on node {node_oid} [{worker_started}]',
-                                    func=hltype(self.check_and_apply),
+                                    func=hltype(self._check_and_apply),
                                     node_oid=hlid(node_oid),
                                     worker_id=hlid(worker_id),
                                     worker_started=worker_started)
@@ -192,13 +209,13 @@ class WebClusterMonitor(object):
                                     raise
                                 self.log.info(
                                     '{func} No Transport {transport_id} currently running for Web cluster worker {worker_id}: starting transport ..',
-                                    func=hltype(self.check_and_apply),
+                                    func=hltype(self._check_and_apply),
                                     worker_id=hlid(worker_id),
                                     transport_id=hlid(transport_id))
                             else:
                                 self.log.debug(
                                     '{func} Ok, transport {transport_id} already running on Web cluster worker {worker_id}',
-                                    func=hltype(self.check_and_apply),
+                                    func=hltype(self._check_and_apply),
                                     worker_id=hlid(worker_id),
                                     transport_id=hlid(transport_id))
 
@@ -239,7 +256,7 @@ class WebClusterMonitor(object):
                                         transport_id)
                                     self.log.info(
                                         '{func} Transport {transport_id} started on Web cluster worker {worker_id} [{transport_started}]',
-                                        func=hltype(self.check_and_apply),
+                                        func=hltype(self._check_and_apply),
                                         worker_id=hlid(worker_id),
                                         transport_id=hlid(transport_id),
                                         transport_started=transport_started)
@@ -275,14 +292,14 @@ class WebClusterMonitor(object):
                                             raise
                                         self.log.info(
                                             '{func} No Web service currently running on path "{path}" for Web cluster worker {worker_id} web transport {transport_id}: starting web service ..',
-                                            func=hltype(self.check_and_apply),
+                                            func=hltype(self._check_and_apply),
                                             path=hlval(path),
                                             worker_id=hlid(worker_id),
                                             transport_id=hlid(transport_id))
                                     else:
                                         self.log.debug(
                                             '{func} Ok, web service on path "{path}" is already running for Web cluster worker {worker_id} web transport {transport_id}',
-                                            func=hltype(self.check_and_apply),
+                                            func=hltype(self._check_and_apply),
                                             path=hlval(path),
                                             worker_id=hlid(worker_id),
                                             transport_id=hlid(transport_id))
@@ -304,7 +321,7 @@ class WebClusterMonitor(object):
                                                 node_oid, worker_id, transport_id, path, webservice_config)
                                             self.log.info(
                                                 '{func} Web service started on transport {transport_id} and path "{path}" [{webservice_started}]',
-                                                func=hltype(self.check_and_apply),
+                                                func=hltype(self._check_and_apply),
                                                 transport_id=hlid(transport_id),
                                                 path=hlval(path),
                                                 webservice_started=webservice_started)
@@ -322,7 +339,7 @@ class WebClusterMonitor(object):
                                     if arealm and arealm.status == ApplicationRealmStatus.RUNNING and arealm.workergroup_oid:
                                         self.log.debug(
                                             '{func} node {node_id} - worker {worker_id} - webcluster "{webcluster_name}": backend router workergroup {workergroup_oid} is associated with this frontend web cluster for application realm "{arealm_name}"',
-                                            func=hltype(self.check_and_apply),
+                                            func=hltype(self._check_and_apply),
                                             node_id=hlval(str(node.node_id)),
                                             worker_id=hlval(worker_id),
                                             webcluster_name=hlval(webcluster.name),
@@ -334,7 +351,7 @@ class WebClusterMonitor(object):
                         workers[wk] = worker
                 else:
                     self.log.warn('{func} Web cluster node {node_oid} not running [status={status}]',
-                                  func=hltype(self.check_and_apply),
+                                  func=hltype(self._check_and_apply),
                                   node_oid=hlid(node_oid),
                                   status=hl(node.status if node else 'offline'))
                     is_running_completely = False
@@ -364,11 +381,13 @@ class WebClusterMonitor(object):
                 status = worker['status'].upper()
             else:
                 status = 'MISSING'
-            self.log.info('{func} {node_oid} {worker_id} {status}',
-                          func=hltype(self.check_and_apply),
-                          worker_id=hlid(worker_id),
-                          node_oid=hlid(node_oid),
-                          status=hlval(status))
+            self.log.info(
+                '{func} webcluster {webcluster_oid} worker {worker_id} on node {node_oid} has status {status}',
+                func=hltype(self._check_and_apply),
+                worker_id=hlid(worker_id),
+                node_oid=hlid(node_oid),
+                webcluster_oid=hlid(self._webcluster_oid),
+                status=hlval(status))
 
         if is_running_completely:
             color = 'green'
@@ -378,10 +397,10 @@ class WebClusterMonitor(object):
             action = 'check & apply run finished with problems left'
 
         self._check_and_apply_in_progress = False
-        self.log.debug('{func} {action} for webcluster {webcluster}!',
-                       action=hl(action, color=color, bold=True),
-                       func=hltype(self.check_and_apply),
-                       webcluster=hlid(self._webcluster_oid))
+        self.log.info('{func} {action} for webcluster {webcluster}!',
+                      action=hl(action, color=color, bold=True),
+                      func=hltype(self._check_and_apply),
+                      webcluster=hlid(self._webcluster_oid))
 
 
 class WebClusterManager(object):
@@ -396,19 +415,26 @@ class WebClusterManager(object):
     # publication options for management API events
     _PUBOPTS = PublishOptions(acknowledge=True)
 
-    # map of allowed web services
+    # map of allowed web services, see also crossbar.personality.Personality.WEB_SERVICE_CHECKERS
     _WEB_SERVICE_CHECKERS = {
+        # none
         'path': checkconfig.check_web_path_service_path,
-        'static': checkconfig.check_web_path_service_static,
-        'archive': archive.RouterWebServiceArchive.check,
-        'json': checkconfig.check_web_path_service_json,
-        'nodeinfo': checkconfig.check_web_path_service_nodeinfo,
         'redirect': checkconfig.check_web_path_service_redirect,
+        # resource
         'reverseproxy': checkconfig.check_web_path_service_reverseproxy,
-        'websocket': checkconfig.check_web_path_service_websocket,
-        'websocket-reverseproxy': checkconfig.check_web_path_service_websocket_reverseproxy,
+        'nodeinfo': checkconfig.check_web_path_service_nodeinfo,
+        'json': checkconfig.check_web_path_service_json,
         'cgi': checkconfig.check_web_path_service_cgi,
         'wsgi': checkconfig.check_web_path_service_wsgi,
+        'static': checkconfig.check_web_path_service_static,
+        'websocket': checkconfig.check_web_path_service_websocket,
+        'websocket-reverseproxy': checkconfig.check_web_path_service_websocket_reverseproxy,
+        # longpoll
+        'caller': checkconfig.check_web_path_service_caller,
+        'publisher': checkconfig.check_web_path_service_publisher,
+        # webhook
+        'archive': archive.RouterWebServiceArchive.check,
+        'wap': wap.RouterWebServiceWap.check,
     }
 
     def __init__(self, session, globaldb, globalschema, db, schema, reactor=None):
@@ -478,24 +504,35 @@ class WebClusterManager(object):
         :return:
         """
         assert self._started is None, 'cannot start web cluster manager - already running!'
+        assert self._prefix is None
 
-        regs = yield self._session.register(self, prefix=prefix, options=RegisterOptions(details_arg='details'))
+        self._started = time_ns()
+
+        # crossbarfabriccenter.mrealm.webcluster
+        self._prefix = prefix[:-1] if prefix.endswith('.') else prefix
+
+        regs = yield self._session.register(self,
+                                            prefix='{}.'.format(self._prefix),
+                                            options=RegisterOptions(details_arg='details'))
         procs = [reg.procedure for reg in regs]
-        self.log.debug('Mrealm controller {api} registered management procedures [{func}]:\n\n{procs}\n',
-                       api=hl('Web cluster manager API', color='green', bold=True),
-                       func=hltype(self.start),
-                       procs=hl(pformat(procs), color='white', bold=True))
+        self.log.debug(
+            'Web cluster manager {api} registered management procedures using prefix "{prefix}" [{func}]:\n\n{procs}\n',
+            api=hl('Web cluster manager API', color='green', bold=True),
+            func=hltype(self.start),
+            prefix=hlval(self._prefix),
+            procs=hl(pformat(procs), color='white', bold=True))
 
         # start all web cluster monitors ..
         cnt_started = 0
         cnt_skipped = 0
+        dl = []
         with self.db.begin() as txn:
             webcluster_oids = self.schema.webclusters.select(txn, return_values=False)
             for webcluster_oid in webcluster_oids:
                 webcluster = self.schema.webclusters[txn, webcluster_oid]
                 if webcluster.status in [cluster.STATUS_STARTING, cluster.STATUS_RUNNING]:
                     monitor = WebClusterMonitor(self, webcluster_oid)
-                    monitor.start()
+                    dl.append(monitor.start())
                     assert webcluster_oid not in self._monitors
                     self._monitors[webcluster_oid] = monitor
                     cnt_started += 1
@@ -520,11 +557,12 @@ class WebClusterManager(object):
             cnt_skipped=hlval(cnt_skipped),
             func=hltype(self.start))
 
-        self._started = time_ns()
         self.log.info('Web cluster manager ready for management realm {mrealm_oid}! [{func}]',
                       mrealm_oid=hlid(self._mrealm_oid),
                       func=hltype(self.start))
 
+        # return txaio.gather(dl)
+
     @inlineCallbacks
     def stop(self):
         """
@@ -533,8 +571,20 @@ class WebClusterManager(object):
         :return:
         """
         assert self._started > 0, 'cannot stop web cluster manager - currently not running!'
-        yield sleep(0)
+
+        # stop all web cluster monitors ..
+        dl = []
+        for webcluster_oid, webcluster_monitor in self._monitors.items():
+            dl.append(webcluster_monitor.stop())
+            del self._monitors[webcluster_oid]
         self._started = None
+        self.log.info(
+            'Ok, web cluster manager for management realm {mrealm_oid} stopped ({cnt_stopped} monitors stopped) [{func}]',
+            mrealm_oid=hlid(self._mrealm_oid),
+            cnt_stopped=len(dl),
+            func=hltype(self.start))
+
+        # return txaio.gather(dl)
 
     @wamp.register(None, check_types=True)
     def list_webclusters(self,
@@ -1048,27 +1098,6 @@ class WebClusterManager(object):
 
         return webcluster_stopping
 
-    @wamp.register(None, check_types=True)
-    def stat_webcluster(self, webcluster_oid: str, details: Optional[CallDetails] = None) -> dict:
-        """
-        *NOT YET IMPLEMENTED*
-
-        Get current status and statistics for given webcluster.
-
-        :param webcluster_oid: The web cluster to return status and statistics for.
-
-        :return: Current status and statistics for given webcluster.
-        """
-        assert type(webcluster_oid) == str
-        assert details is None or isinstance(details, CallDetails)
-
-        self.log.info('{func}(webcluster_oid={webcluster_oid}, details={details})',
-                      func=hltype(self.stat_webcluster),
-                      webcluster_oid=hlid(webcluster_oid),
-                      details=details)
-
-        raise NotImplementedError()
-
     @wamp.register(None, check_types=True)
     def list_webcluster_nodes(self,
                               webcluster_oid: str,
@@ -1733,31 +1762,3 @@ class WebClusterManager(object):
             assert webservice
 
         return webservice.marshal()
-
-    @wamp.register(None, check_types=True)
-    def stat_webcluster_service(self,
-                                webcluster_oid: str,
-                                webservice_oid: str,
-                                details: Optional[CallDetails] = None) -> dict:
-        """
-        *NOT YET IMPLEMENTED*
-
-        Return current status and statistics for the given web service.
-
-        :param webcluster_oid: The web cluster running the web service to return status and statistics for.
-        :param webservice_oid: The web service to return status and statistics for.
-
-        :return: Current status and statistics information for the web service.
-        """
-        assert type(webcluster_oid) == str
-        assert type(webservice_oid) == str
-        assert details is None or isinstance(details, CallDetails)
-
-        self.log.info(
-            '{klass}.stat_webcluster_service(webcluster_oid={webcluster_oid}, webservice_oid={webservice_oid}, details={details})',
-            webcluster_oid=hlid(webcluster_oid),
-            webservice_oid=hlid(webservice_oid),
-            func=hltype(self.stat_webcluster_service),
-            details=details)
-
-        raise NotImplementedError()
