@@ -16,10 +16,6 @@ import six
 
 import zlmdb
 
-import nacl
-import nacl.signing
-import nacl.encoding
-
 from twisted.internet import defer
 from twisted.internet.defer import inlineCallbacks, returnValue, DeferredList, Deferred
 from twisted.internet.task import LoopingCall
@@ -35,7 +31,7 @@ from autobahn.wamp.exception import ApplicationError
 from autobahn.twisted.wamp import ApplicationSession
 
 from crossbar._util import hl, hlid, hlval, hltype
-from crossbar.common.key import _read_node_key, _read_release_key
+from crossbar.common.key import _read_release_key
 
 from cfxdb.mrealmschema import MrealmSchema
 from cfxdb.globalschema import GlobalSchema
@@ -168,10 +164,6 @@ class MrealmController(ApplicationSession):
         # Release (public) key
         self._release_pubkey_hex = _read_release_key()['hex']
 
-        # Node key
-        self._node_key_hex = _read_node_key('.', private=True)['hex']
-        self._node_key = nacl.signing.SigningKey(self._node_key_hex, encoder=nacl.encoding.HexEncoder)
-
         assert 'mrealm' in config.extra and type(config.extra['mrealm'] == str)
         self._mrealm_oid = uuid.UUID(config.extra['mrealm'])
 
@@ -187,7 +179,8 @@ class MrealmController(ApplicationSession):
         assert maxsize and type(maxsize) in six.integer_types
         assert maxsize >= 2**20 and maxsize < 2**30 * 10  # 1 MB - 10 GB maximum size
 
-        self.gdb = zlmdb.Database(dbpath=dbfile, maxsize=maxsize, readonly=False, sync=True)
+        # self.gdb = zlmdb.Database(dbpath=dbfile, maxsize=maxsize, readonly=False, sync=True, context=self)
+        self.gdb = zlmdb.Database.open(dbpath=dbfile, maxsize=maxsize, readonly=False, sync=True, context=self)
         self.gdb.__enter__()
         self.gschema: GlobalSchema = GlobalSchema.attach(self.gdb)
 
@@ -207,7 +200,8 @@ class MrealmController(ApplicationSession):
         assert maxsize and type(maxsize) in six.integer_types
         assert maxsize >= 2**20 and maxsize < 2**30 * 10  # 1 MB - 10 GB maximum size
 
-        self.db = zlmdb.Database(dbpath=dbfile, maxsize=maxsize, readonly=False, sync=True)
+        # self.db = zlmdb.Database(dbpath=dbfile, maxsize=maxsize, readonly=False, sync=True, context=self)
+        self.db = zlmdb.Database.open(dbpath=dbfile, maxsize=maxsize, readonly=False, sync=True, context=self)
         self.db.__enter__()
         self.schema = MrealmSchema.attach(self.db)
 
@@ -743,7 +737,7 @@ class MrealmController(ApplicationSession):
         assert type(heartbeat_seq) == int
 
         heartbeat_pubkey = heartbeat.get('pubkey', None)
-        assert type(heartbeat_pubkey) == str and len(heartbeat_pubkey) == 64
+        assert heartbeat_pubkey is None or (type(heartbeat_pubkey) == str and len(heartbeat_pubkey) == 64)
 
         heartbeat_workers = heartbeat.get('workers', {})
         assert type(heartbeat_workers) == dict
