@@ -24,9 +24,9 @@ from crossbar.worker.testee import WebSocketTesteeController
 from crossbar.worker.proxy import ProxyController, ProxyWorkerProcess
 from crossbar.webservice import base
 from crossbar.webservice import wsgi, rest, longpoll, websocket, misc, static, archive, wap, catalog
-from crossbar.interfaces import IRealmStore, IRealmInventory
+from crossbar.interfaces import IRealmStore, IInventory
 from crossbar.router.realmstore import RealmStoreMemory
-from crossbar.router.realminventory import RealmInventory
+from crossbar.router.inventory import Inventory
 
 
 def do_nothing(*args, **kw):
@@ -153,9 +153,34 @@ def create_realm_store(personality, factory, config) -> IRealmStore:
     return store
 
 
-def create_realm_inventory(personality, factory, config) -> IRealmInventory:
+def create_realm_inventory(personality, factory, config) -> IInventory:
     """
 
+    .. code-block:: json
+
+        {
+            "version": 2,
+            "workers": [
+                {
+                    "type": "router",
+                    "realms": [
+                        {
+                            "name": "realm1",
+                            "inventory": {
+                                "type": "wamp.eth",
+                                "catalogs": [
+                                    {
+                                        "name": "pydefi",
+                                        "filename": "../schema/trading.bfbs"
+                                    }
+                                ]
+                            }
+                        }
+                    ]
+                }
+            ]
+        }
+
     :param personality: Node personality
     :type personality: :class:`crossbar.personality`
 
@@ -163,11 +188,21 @@ def create_realm_inventory(personality, factory, config) -> IRealmInventory:
     :type factory: :class:`crossbar.router.router.RouterFactory`
 
     :param config: FbsRepository configuration
-    :type config: dict
+    :type config: dict, for example:
+        .. code-block:: json
+            {
+                "type": "wamp.eth",
+                "catalogs": [
+                    {
+                        "name": "pydefi",
+                        "filename": "../schema/trading.bfbs"
+                    }
+                ]
+            }
 
     :return: A new realm inventory object.
     """
-    inventory = RealmInventory(personality, factory, config)
+    inventory = Inventory.from_config(personality, factory, config)
     return inventory
 
 
