@@ -22,6 +22,7 @@ from autobahn import util
 from autobahn.util import hltype, hlid, hlval
 from autobahn.wamp.types import Accept, Deny, HelloDetails, Challenge, TransportDetails
 from autobahn.wamp.exception import ApplicationError
+from autobahn.wamp.message import identity_realm_name_category
 
 from crossbar.router.auth.pending import PendingAuth
 from crossbar.interfaces import IRealmContainer, IPendingAuth
@@ -59,13 +60,29 @@ class PendingAuthCryptosign(PendingAuth):
         self._challenge: Optional[bytes] = None
         self._expected_signed_message: Optional[bytes] = None
 
-        # create a map: pubkey -> authid
-        # this is to allow clients to authenticate without specifying an authid
-        if config['type'] == 'static':
+        self._trustroot = None
+        if self._config['type'] == 'static' and 'trustroot' in self._config:
+            self._trustroot = self._config['trustroot']
+            self._trustroot_name_category = identity_realm_name_category(self._trustroot)
+
+            # this is already checked in checkconfig
+            assert self._trustroot_name_category in ['eth', 'ens', 'reverse_ens']
+            self.log.info('{func} using trustroot {trustroot_name_category} "{trustroot}" from static config',
+                          trustroot=hlid(self._trustroot),
+                          trustroot_name_category=hlval(self._trustroot_name_category, color='green'),
+                          func=hltype(self))
+
+        # create a map `pubkey -> authid` from `config['principals']`, this is to allow clients to
+        # authenticate without specifying an authid
+        self._pubkey_to_authid = None
+        if self._config['type'] == 'static' and 'principals' in self._config:
             self._pubkey_to_authid = {}
             for authid, principal in self._config.get('principals', {}).items():
                 for pubkey in principal['authorized_keys']:
                     self._pubkey_to_authid[pubkey] = authid
+            self.log.info('{func} using principals ({pubkeys_cnt} pubkeys loaded)',
+                          pubkeys_cnt=hlval(len(self._pubkey_to_authid), color='green'),
+                          func=hltype(self))
 
     def _compute_challenge(self, requested_channel_binding: Optional[str]) -> Dict[str, Any]:
         self._challenge = os.urandom(32)
@@ -165,7 +182,45 @@ class PendingAuthCryptosign(PendingAuth):
                 self._verify_key = VerifyKey(pubkey, encoder=nacl.encoding.HexEncoder)
 
                 extra = self._compute_challenge(requested_channel_binding)
-                return Challenge(self._authmethod, extra)
+
+                if 'challenge' in details.authextra and details.authextra['challenge']:
+                    challenge_raw = binascii.a2b_hex(details.authextra['challenge'])
+                    if requested_channel_binding == 'tls-unique':
+                        data = util.xor(challenge_raw, self._channel_id)
+                    else:
+                        data = challenge_raw
+
+                    # sign the client challenge with our node private Ed25519 key on node controller
+                    signature_d = self._realm_container.get_controller_session().call('crossbar.sign', data)
+
+                    def _on_sign_ok(signature):
+                        # return the concatenation of the signature and the message signed (96 bytes)
+                        extra['signature'] = binascii.b2a_hex(signature).decode() + binascii.b2a_hex(data).decode()
+
+                    signature_d.addCallback(_on_sign_ok)
+
+                    # get node public key from node controller
+                    pubkey_d = self._realm_container.get_controller_session().call('crossbar.get_public_key')
+
+                    def _on_pubkey_ok(pubkey):
+                        # return router public key
+                        extra['pubkey'] = pubkey
+
+                    pubkey_d.addCallback(_on_pubkey_ok)
+
+                    # FIXME: add router certificate
+                    # FIXME: add router trustroot
+
+                    d = txaio.gather([signature_d, pubkey_d])
+
+                    def _on_final(_):
+                        return Challenge(self._authmethod, extra)
+
+                    d.addCallback(_on_final)
+                    return d
+
+                else:
+                    return Challenge(self._authmethod, extra)
 
             else:
                 self.log.warn(
@@ -220,7 +275,8 @@ class PendingAuthCryptosign(PendingAuth):
                     self._verify_key = VerifyKey(principal['pubkey'], encoder=nacl.encoding.HexEncoder)
 
                     extra = self._compute_challenge(requested_channel_binding)
-                    d.callback(Challenge(self._authmethod, extra))
+                    challenge = Challenge(self._authmethod, extra)
+                    d.callback(challenge)
 
                 def on_authenticate_error(_error):
                     self.log.debug(
