@@ -9,6 +9,7 @@ from twisted.trial import unittest
 from twisted.internet import defer
 
 import txaio
+
 txaio.use_twisted()  # noqa
 
 from crossbar.router.role import RouterRoleStaticAuth
@@ -29,7 +30,7 @@ class MockRealmContainer(object):
         return realm == self._realm
 
     def has_role(self, realm, role):
-        return realm == self._realm and (role == 'trusted' or role in self._roles)
+        return realm == self._realm and role in self._roles
 
     def get_service_session(self, realm, role):
         assert realm == self._realm, 'realm must be "{}", but was "{}"'.format(self._realm, realm)
@@ -43,8 +44,7 @@ class TestDynamicAuth(unittest.TestCase):
         We pass along the authextra to a dynamic authenticator
         """
         session = Mock()
-        session._transport._transport_info = {}
-        session
+        session._transport.transport_details = types.TransportDetails()
 
         def fake_call(method, *args, **kw):
             realm, authid, details = args
@@ -69,6 +69,8 @@ class TestDynamicAuth(unittest.TestCase):
         config = {
             "type": "dynamic",
             "authenticator": "foo.auth_a_doodle",
+            "authenticator-realm": "realm",
+            "authenticator-role": "myauth_role"
         }
         extra = {
             "foo": "bar",
@@ -77,10 +79,10 @@ class TestDynamicAuth(unittest.TestCase):
         details.authextra = extra
 
         pending_session_id = 1
-        transport_info = {}
-        realm_container = MockRealmContainer("realm", ["some_role"], session)
+        transport_details = types.TransportDetails()
+        realm_container = MockRealmContainer("realm", ["some_role", "myauth_role"], session)
 
-        auth = cryptosign.PendingAuthCryptosign(pending_session_id, transport_info, realm_container, config)
+        auth = cryptosign.PendingAuthCryptosign(pending_session_id, transport_details, realm_container, config)
         val = yield auth.hello("realm", details)
 
         self.assertTrue(isinstance(val, types.Challenge))
@@ -94,7 +96,7 @@ class TestDynamicAuth(unittest.TestCase):
         We pass along the authextra to a dynamic authenticator
         """
         session = Mock()
-        session._transport._transport_info = {}
+        session._transport.transport_details = types.TransportDetails()
 
         def fake_call(method, *args, **kw):
             realm, authid, details = args
@@ -120,6 +122,8 @@ class TestDynamicAuth(unittest.TestCase):
         config = {
             "type": "dynamic",
             "authenticator": "foo.auth_a_doodle",
+            "authenticator-realm": "realm",
+            "authenticator-role": "myauth_role"
         }
         extra = {
             "foo": "bar",
@@ -129,10 +133,10 @@ class TestDynamicAuth(unittest.TestCase):
         details.authextra = extra
 
         pending_session_id = 1
-        transport_info = {}
-        realm_container = MockRealmContainer("realm", ["some_role"], session)
+        transport_details = types.TransportDetails()
+        realm_container = MockRealmContainer("realm", ["some_role", "myauth_role"], session)
 
-        auth = wampcra.PendingAuthWampCra(pending_session_id, transport_info, realm_container, config)
+        auth = wampcra.PendingAuthWampCra(pending_session_id, transport_details, realm_container, config)
         val = yield auth.hello("realm", details)
 
         self.assertTrue(isinstance(val, types.Challenge))
@@ -146,7 +150,7 @@ class TestDynamicAuth(unittest.TestCase):
         We pass along the authextra to a dynamic authenticator
         """
         session = Mock()
-        session._transport._transport_info = {}
+        session._transport.transport_details = types.TransportDetails()
 
         def fake_call(method, *args, **kw):
             realm, authid, details = args
@@ -172,6 +176,8 @@ class TestDynamicAuth(unittest.TestCase):
         config = {
             "type": "dynamic",
             "authenticator": "foo.auth_a_doodle",
+            "authenticator-realm": "realm",
+            "authenticator-role": "myauth_role"
         }
         extra = {
             "foo": "bar",
@@ -181,10 +187,10 @@ class TestDynamicAuth(unittest.TestCase):
         details.authextra = extra
 
         pending_session_id = 1
-        transport_info = {}
-        realm_container = MockRealmContainer("realm", ["some_role"], session)
+        transport_details = types.TransportDetails(channel_id={'tls-unique': b'anything'}, peer_cert={'some': 'thing'})
+        realm_container = MockRealmContainer("realm", ["some_role", "myauth_role"], session)
 
-        auth = tls.PendingAuthTLS(pending_session_id, transport_info, realm_container, config)
+        auth = tls.PendingAuthTLS(pending_session_id, transport_details, realm_container, config)
         val = yield auth.hello("realm", details)
 
         self.assertTrue(isinstance(val, types.Accept))
@@ -197,7 +203,7 @@ class TestDynamicAuth(unittest.TestCase):
         We pass along the authextra to a dynamic authenticator
         """
         session = Mock()
-        session._transport._transport_info = {}
+        session._transport.transport_details = types.TransportDetails()
 
         def fake_call(method, *args, **kw):
             realm, authid, details = args
@@ -223,6 +229,8 @@ class TestDynamicAuth(unittest.TestCase):
         config = {
             "type": "dynamic",
             "authenticator": "foo.auth_a_doodle",
+            "authenticator-realm": "realm",
+            "authenticator-role": "myauth_role"
         }
         extra = {
             "foo": "bar",
@@ -232,10 +240,10 @@ class TestDynamicAuth(unittest.TestCase):
         details.authextra = extra
 
         pending_session_id = 1
-        transport_info = {}
-        realm_container = MockRealmContainer("realm", ["some_role"], session)
+        transport_details = types.TransportDetails()
+        realm_container = MockRealmContainer("realm", ["some_role", "myauth_role"], session)
 
-        auth = anonymous.PendingAuthAnonymous(pending_session_id, transport_info, realm_container, config)
+        auth = anonymous.PendingAuthAnonymous(pending_session_id, transport_details, realm_container, config)
         val = yield auth.hello("realm", details)
 
         self.assertTrue(isinstance(val, types.Accept))
@@ -248,7 +256,7 @@ class TestDynamicAuth(unittest.TestCase):
         We pass along the authextra to a dynamic authenticator
         """
         session = Mock()
-        session._transport._transport_info = {}
+        session._transport.transport_details = types.TransportDetails()
 
         def fake_call(method, *args, **kw):
             realm, authid, details = args
@@ -274,6 +282,8 @@ class TestDynamicAuth(unittest.TestCase):
         config = {
             "type": "dynamic",
             "authenticator": "foo.auth_a_doodle",
+            "authenticator-realm": "realm",
+            "authenticator-role": "myauth_role"
         }
         extra = {
             "foo": "bar",
@@ -283,10 +293,10 @@ class TestDynamicAuth(unittest.TestCase):
         details.authextra = extra
 
         pending_session_id = 1
-        transport_info = {}
-        realm_container = MockRealmContainer("realm", ["some_role"], session)
+        transport_details = types.TransportDetails()
+        realm_container = MockRealmContainer("realm", ["some_role", "myauth_role"], session)
 
-        auth = ticket.PendingAuthTicket(pending_session_id, transport_info, realm_container, config)
+        auth = ticket.PendingAuthTicket(pending_session_id, transport_details, realm_container, config)
         val = yield auth.hello("realm", details)
 
         self.assertTrue(isinstance(val, types.Challenge))
