@@ -635,6 +635,129 @@ class CmdRemoveWebClusterNode(CmdRemove):
         return self._post(session, result)
 
 
+class CmdRemoveArealmPrincipal(CmdRemove):
+    def __init__(self, arealm, principal):
+        CmdRemove.__init__(self)
+        self.arealm = arealm
+        self.principal = principal
+
+    async def run(self, session):
+        self._pre(session)
+
+        try:
+            arealm_oid = uuid.UUID(self.arealm)
+        except:
+            arealm = await session.call('crossbarfabriccenter.mrealm.arealm.get_arealm_by_name', self.arealm)
+            arealm_oid = arealm['oid']
+        else:
+            arealm_oid = str(arealm_oid)
+
+        try:
+            principal_oid = uuid.UUID(self.principal)
+        except:
+            principal = await session.call('crossbarfabriccenter.mrealm.arealm.get_principal_by_name', self.arealm)
+            principal_oid = principal['oid']
+        else:
+            principal_oid = str(principal_oid)
+
+        result = await session.call('crossbarfabriccenter.mrealm.arealm.remove_principal', arealm_oid, principal_oid)
+
+        return self._post(session, result)
+
+
+class CmdRemoveArealmPrincipalCredential(CmdRemove):
+    def __init__(self, arealm, principal, credential):
+        CmdRemove.__init__(self)
+        self.arealm = arealm
+        self.principal = principal
+        self.credential = credential
+
+    async def run(self, session):
+        self._pre(session)
+
+        try:
+            arealm_oid = uuid.UUID(self.arealm)
+        except:
+            arealm = await session.call('crossbarfabriccenter.mrealm.arealm.get_arealm_by_name', self.arealm)
+            arealm_oid = arealm['oid']
+        else:
+            arealm_oid = str(arealm_oid)
+
+        try:
+            principal_oid = uuid.UUID(self.principal)
+        except:
+            principal = await session.call('crossbarfabriccenter.mrealm.arealm.get_principal_by_name', self.arealm)
+            principal_oid = principal['oid']
+        else:
+            principal_oid = str(principal_oid)
+
+        credential_oid = uuid.UUID(self.credential)
+
+        result = await session.call('crossbarfabriccenter.mrealm.arealm.remove_principal_credential', arealm_oid,
+                                    principal_oid, credential_oid)
+
+        return self._post(session, result)
+
+
+class CmdRemoveRolePermission(CmdRemove):
+    def __init__(self, role, path):
+        CmdRemove.__init__(self)
+        self.role = role
+        self.path = path
+
+    async def run(self, session):
+        self._pre(session)
+
+        try:
+            role_oid = uuid.UUID(self.role)
+        except:
+            role = await session.call('crossbarfabriccenter.mrealm.arealm.get_role_by_name', self.role)
+            role_oid = role['oid']
+        else:
+            role_oid = str(role_oid)
+
+        permission_oids = await session.call('crossbarfabriccenter.mrealm.arealm.get_role_permissions_by_uri',
+                                             role_oid, self.path)
+
+        results = []
+        for permission_oid in permission_oids:
+            result = await session.call('crossbarfabriccenter.mrealm.arealm.remove_role_permission', role_oid,
+                                        permission_oid)
+            results.append(result)
+
+        return self._post(session, result)
+
+
+class CmdRemoveArealmRole(CmdRemove):
+    def __init__(self, arealm, role):
+        CmdRemove.__init__(self)
+        self.arealm = arealm
+        self.role = role
+
+    async def run(self, session):
+        self._pre(session)
+
+        try:
+            arealm_oid = uuid.UUID(self.arealm)
+        except:
+            arealm = await session.call('crossbarfabriccenter.mrealm.arealm.get_arealm_by_name', self.arealm)
+            arealm_oid = arealm['oid']
+        else:
+            arealm_oid = str(arealm_oid)
+
+        try:
+            role_oid = uuid.UUID(self.role)
+        except:
+            role = await session.call('crossbarfabriccenter.mrealm.arealm.get_role_by_name', self.role)
+            role_oid = role['oid']
+        else:
+            role_oid = str(role_oid)
+
+        result = await session.call('crossbarfabriccenter.mrealm.arealm.remove_arealm_role', arealm_oid, role_oid)
+
+        return self._post(session, result)
+
+
 class CmdDelete(Cmd):
     def __init__(self):
         Cmd.__init__(self)
@@ -654,6 +777,47 @@ class CmdDeleteManagementRealm(CmdDelete):
         return self._post(session, result)
 
 
+class CmdDeleteApplicationRealm(CmdDelete):
+    def __init__(self, realm, cascade):
+        CmdDelete.__init__(self)
+        self.realm = realm
+        self.cascade = cascade
+
+    async def run(self, session):
+        self._pre(session)
+
+        try:
+            arealm_oid = uuid.UUID(self.realm)
+        except:
+            arealm = await session.call('crossbarfabriccenter.mrealm.arealm.get_arealm_by_name', self.realm)
+            arealm_oid = arealm['oid']
+        else:
+            arealm_oid = str(arealm_oid)
+
+        result = await session.call('crossbarfabriccenter.mrealm.arealm.delete_arealm', arealm_oid)
+        return self._post(session, result)
+
+
+class CmdDeleteRole(CmdDelete):
+    def __init__(self, role):
+        CmdDelete.__init__(self)
+        self.role = role
+
+    async def run(self, session):
+        self._pre(session)
+
+        try:
+            role_oid = uuid.UUID(self.role)
+        except:
+            role = await session.call('crossbarfabriccenter.mrealm.arealm.get_role_by_name', self.role)
+            role_oid = role['oid']
+        else:
+            role_oid = str(role_oid)
+
+        result = await session.call('crossbarfabriccenter.mrealm.arealm.delete_role', role_oid)
+        return self._post(session, result)
+
+
 class CmdDeleteRouterCluster(CmdDelete):
     """
     MREALM: delete routercluster by UUID.
