@@ -29,6 +29,7 @@ import _cffi_backend  # noqa
 
 # import and select network framework in txaio _before_ any further crossbar.shell imports
 import txaio
+
 txaio.use_twisted()  # noqa
 
 from cfxdb.exporter import Exporter
@@ -66,8 +67,9 @@ class Config(object):
     Command configuration object where we collect all the parameters,
     options etc for later processing.
     """
-    def __init__(self, app, profile, realm, role):
+    def __init__(self, app, dotdir, profile, realm, role):
         self.app = app
+        self.dotdir = dotdir
         self.profile = profile
         self.realm = realm
         self.role = role
@@ -75,10 +77,16 @@ class Config(object):
         self.resource = None
 
     def __str__(self):
-        return u'Config(resource_type={}, resource={})'.format(self.resource_type, self.resource)
+        return 'Config(resource_type={}, resource={})'.format(self.resource_type, self.resource)
 
 
 @click.group(help="Crossbar.io Command Line", invoke_without_command=True)
+@click.option(
+    '--dotdir',
+    envvar='CBF_DOTDIR',
+    default=None,
+    help="Set the dot directory (with config and profile) to be used",
+)
 @click.option(
     '--profile',
     envvar='CBF_PROFILE',
@@ -99,11 +107,11 @@ class Config(object):
 )
 @click.option('--debug', is_flag=True, help='Enable debug output')
 @click.pass_context
-def cli(ctx, profile, realm, role, debug):
+def cli(ctx, dotdir, profile, realm, role, debug):
     if debug:
         txaio.start_logging(level='info')
 
-    ctx.obj = Config(_app, profile, realm, role)
+    ctx.obj = Config(_app, dotdir, profile, realm, role)
 
     # Allowing a command group to specifiy a default subcommand can be done using
     # https://github.com/click-contrib/click-default-group
@@ -373,11 +381,12 @@ def cmd_show_domain_license(ctx):
 
 @cmd_show.command(name='database', help='open and show embedded database details')
 @click.argument('dbpath')
+@click.option('--include-slots/--no-include-slots', default=False, type=bool, help='show database slots')
 @click.pass_context
-def cmd_show_database(ctx, dbpath):
+def cmd_show_database(ctx, dbpath, include_slots):
     exporter = Exporter(dbpath)
-    exporter.print_slots()
-    exporter.print_stats()
+    exporter.print_config()
+    exporter.print_stats(include_slots=include_slots)
 
 
 @cli.group(name='export', help='export resources')
@@ -390,7 +399,9 @@ def cmd_export(ctx):
 @click.argument('dbpath')
 @click.option('--filename')
 @click.option('--include-indexes/--no-include-indexes', default=False, type=bool, help='export index tables')
-@click.option('--include-schemata', type=str)
+@click.option('--include-schemata',
+              type=str,
+              help='list of schemata to export (meta, globalschema, mrealmschema, xbr, xbrmm, xbrnetwork)')
 @click.option('--exclude-tables', type=str)
 @click.option('--use-json/--no-use-json', default=False, type=bool)
 @click.option('--use-binary-hex-encoding/--no-use-binary-hex-encoding', default=False, type=bool)
@@ -682,6 +693,15 @@ def cmd_remove_routeercluster_workergroup(ctx, cluster, workergroup):
     ctx.obj.app.run_context(ctx, cmd)
 
 
+@cmd_remove.command(name='webcluster-node', help='remove a node from a webcluster')
+@click.argument('cluster')
+@click.argument('node')
+@click.pass_context
+def cmd_remove_webcluster_node(ctx, cluster, node):
+    cmd = command.CmdRemoveWebClusterNode(cluster, node)
+    ctx.obj.app.run_context(ctx, cmd)
+
+
 @cmd_remove.command(name='webcluster-service', help='remove a service from a webcluster')
 @click.argument('cluster')
 @click.argument('path')
@@ -691,12 +711,40 @@ def cmd_remove_webcluster_service(ctx, cluster, path):
     ctx.obj.app.run_context(ctx, cmd)
 
 
-@cmd_remove.command(name='webcluster-node', help='remove a node from a webcluster')
-@click.argument('cluster')
-@click.argument('node')
+@cmd_remove.command(name='arealm-principal', help='remove a principal from an application realm')
+@click.argument('arealm')
+@click.argument('principal')
 @click.pass_context
-def cmd_remove_webcluster_node(ctx, cluster, node):
-    cmd = command.CmdRemoveWebClusterNode(cluster, node)
+def cmd_remove_arealm_principal(ctx, arealm, principal):
+    cmd = command.CmdRemoveArealmPrincipal(arealm, principal)
+    ctx.obj.app.run_context(ctx, cmd)
+
+
+@cmd_remove.command(name='principal-credential', help='remove credentials from a principal on an application realm')
+@click.argument('arealm')
+@click.argument('principal')
+@click.argument('credential')
+@click.pass_context
+def cmd_remove_arealm_principal_credential(ctx, arealm, principal, credential):
+    cmd = command.CmdRemoveArealmPrincipalCredential(arealm, principal, credential)
+    ctx.obj.app.run_context(ctx, cmd)
+
+
+@cmd_remove.command(name='role-permission', help='remove a permission from a role')
+@click.argument('role')
+@click.argument('path')
+@click.pass_context
+def cmd_remove_role_permission(ctx, role, path):
+    cmd = command.CmdRemoveRolePermission(role, path)
+    ctx.obj.app.run_context(ctx, cmd)
+
+
+@cmd_remove.command(name='arealm-role', help='remove a role from an application realm')
+@click.argument('arealm')
+@click.argument('role')
+@click.pass_context
+def cmd_remove_arealm_role(ctx, arealm, role):
+    cmd = command.CmdRemoveArealmRole(arealm, role)
     ctx.obj.app.run_context(ctx, cmd)
 
 
@@ -736,6 +784,23 @@ def cmd_delete_webcluster(ctx, cluster):
     ctx.obj.app.run_context(ctx, cmd)
 
 
+@cmd_delete.command(name='arealm', help='delete an existing application realm')
+@click.argument('arealm')
+@click.option('--cascade', is_flag=True, help='Automatically delete dependent resources of the application realm.')
+@click.pass_context
+def cmd_delete_arealm(ctx, arealm, cascade=False):
+    cmd = command.CmdDeleteApplicationRealm(arealm, cascade)
+    ctx.obj.app.run_context(ctx, cmd)
+
+
+@cmd_delete.command(name='role', help='delete an existing application role')
+@click.argument('role')
+@click.pass_context
+def cmd_delete_role(ctx, role):
+    cmd = command.CmdDeleteRole(role)
+    ctx.obj.app.run_context(ctx, cmd)
+
+
 @cli.group(name='pair', help='pair nodes and devices')
 @click.pass_context
 def cmd_pair(ctx):
@@ -1410,7 +1475,7 @@ def cmd_select(ctx):
 @click.argument('resource')
 @click.pass_context
 def cmd_select_node(ctx, resource):
-    _app.current_resource_type = u'node'
+    _app.current_resource_type = 'node'
     _app.current_resource = resource
     _app.print_selected()
 
@@ -1419,7 +1484,7 @@ def cmd_select_node(ctx, resource):
 @click.argument('resource')
 @click.pass_context
 def cmd_select_worker(ctx, resource):
-    _app.current_resource_type = u'worker'
+    _app.current_resource_type = 'worker'
     _app.current_resource = resource
     _app.print_selected()
 
@@ -1428,7 +1493,7 @@ def cmd_select_worker(ctx, resource):
 @click.argument('resource')
 @click.pass_context
 def cmd_select_transport(ctx, resource):
-    _app.current_resource_type = u'transport'
+    _app.current_resource_type = 'transport'
     _app.current_resource = resource
     _app.print_selected()
 
